function res = c_eval(cid, tim, varargin)
% c_eval   evaluate the classifier C_id on a test image 
%
% res = c_eval(cid, tim, params...)
%
% cid - identifier model trained on a model image (see h_generateCid) 
% tim - test image (may be a pyramid already - see id_prepIm)
% params... - parameters to change (replaces cid.para fields)
%           passed in as a struct and/or as 'name',value pairs
%           see makeParams.
%
% cid includes:
%    hid model
%    model im pyr
%    patch list: for each patch
%       position, size, level
%       distribution param (D)
%       hf original (as extracted in p)
%       hf selected (from expanded list)
%  
%  struct: cid.para = parameter struct
%          cid.pyr = model image (after prep) 
%          cid.p = patch descriptor - [scale, position, size] - (n x 5) 
%                  each row is: [lev xcent ycent xradius yradius] 
%          cid.D = params of same and diff distributions
%                  based on appearance distance (da) and (optionally) 
%                  position dx, dy: D=1x4 or 1x12 where,
%                  D = [sd dd sdx ddx sdy ddy] (sdx,y ddx,y are optional) 
%                  where each sd and dd = [mu v] (mean and gamma) 
%          cid.ord = the order in which to match patches n x 3, 
%                    [patch_index mi cmi] 
%                    where patch_index is the index into cid.{p,D}
%          cid.thresh = #max_patches x 2, where each row = [th_reject th_accept]
%                  after each patch is matched, 
%                  if llrsum <= th_reject then reject 
%                  else if llrsum > th_accept then accept
%                  else continue
%
%     the following fields are for debug only (not used by c_eval) added only
%     if para.dbg>1 :
%          cid.hf = hyper-features (see id_compHyperFeatures) 
%          cid.hid = category-trained id generator model
%
% res struct includes: 
%       res.m = n x 5 - [x y da dx dy]
%          where x,y actual pos of center (base level coords) 
%                dx, dy delta from expected pos
%                da  dist in appearance
%       res.llr = n x {1 or 2} [llr_d llr_p] 
%                 (llr_p optional, not implemented) 
%       res.c = result value: {1,0,-1}; 0=undecided
%
% steps 
%    prep image
%    for each patch of C_id in order:
%        find matching patch, save d, r=[rx ry]
%        compute lls lld -> llr  
%        if th_accept & th_reject are defined, then  apply them, exit if done

  % compute parameters ...
  para = makeParams(cid.para, varargin{:});
  
  % if tim is not a pyramid already, make it...
  tpyr = id_prepImCache(tim, '', para);

  llrsum = 0;
  res.m=[]; res.llr=[]; res.c=0; 
  
  %try to read match info from cache
  s = id_readCache(['match' cid.pyr{1}.name '_' tpyr{1}.name], para);
  if(length(s)==0 | s.match_tag~=para.match_tag)
    % if not, init it to 0
    s.m = zeros(size(cid.p,1),1);
    s.match_tag = para.match_tag;
  end
  
  for pi = 1:size(cid.ord,1) % go through the patches in order
    ii = cid.ord(pi,1); % find patch index in p,hf,etc. 
    
    % check if has already been compute
    if(s.m(ii,1)>0)
      m = s.m(ii,1);
    else % if not,
      % find the best matching patch. m = [xcent ycent da dx dy] 
      m = id_matchPatch(cid.pyr, tpyr, cid.p(ii,1:5), para);
    end
    
    % compute llr score (based on apearance distance d only)
    if(~isinf(m(3)))
      cllr = c_compLLRd(m(3), cid.D(ii,1:4), para);
    else
      cllr=0;
    end
    llrsum = cllr + llrsum;
    
    %save patch info to result
    res.m(pi,:) = m;
    res.llr(pi,:) = cllr;
    
    % check resulting sum against accept, reject thresholds
    if(size(cid.thresh,1)>=pi)
      if(llrsum <= cid.thresh(pi,1))
	res.c=-1;
      elseif(llrsum > cid.thresh(pi,2))
	res.c=1;
      end
    end
  end
  
  % write matches out to cache... 
  id_writeCache(s, ['match' cid.pyr{1}.name '_' tpyr{1}.name], para);
  
  if(para.show_steps)
    mdisp('LLR sum score after', pi, ' patches matched is ', llrsum); 
  end
  