function fio = frameIo(dirpath, pattern, subseqs, specimname, params)
% frameIo  class for reading image or other file sequences and caching computed data
%
% fio = frameIo(dirpath, pattern(o), subseqs(o), specimname(o), params(o))
% creates a frameIo class with all files that have the specified pattern 
%     in directory dirpath. Two types of indexing is supported: indexing into all elements
%     or indexing into defined subsequences. 
%
% opt parameters:
%   subseqs  cell array of int arrays defines subsequence(s) of frames
%   specimname  cell array of filenames defining special images (or data) that is not 
%               part of the normal seqence   
%   params   struct of parameters for file reading/writing functions
%
% Usage. 
%  Accessors:
%   fio.im returns current image/data (current position of the current subsequence)
%   fio{i} returns ith image/data of the current subsequence
%   fio(i) returns ith image/data 
%   fio('str') returns the first image/data whos name begins with str or [fio.prefix str]
%   getIm(fio, ... ) returns the requested image/data and info  
%   getImInfo(fio, ...) like getIm, but returns only image info (like name, size), only for images 
%   getImName(fio, ...) returns only the image/data name (no disk io)
%   getSpec(fio,...) gets special images/data
%   length(fio)  number of elements in the current subsequence
%   fio.params, fio.dirpath, fio.cacheDir returns fio parameters, etc. 
%  Modifiers:
%   fio = setIm(fio, seqnum, i) sets current subsequence to seqnum and current 
%           element within this sequence to i
%   fio = setParams(fio, ...) sets fio parameters struct passed to IO functions
%   fio = addSpec(fio, ...)  adds special image/data 
%   fio = addSubSeq(fio, seq, ind) add a subsequence definition to this 
%           frameIo object
%  Cache:  
%   cacheDir = getCacheDir(fio) returns cache dir
%   fio = setCacheDir(fio, dirPath) sets cache dir 
%   usecache = useCache(fio, readOrWrite) get caching behaviour for read or write
%   fio = setCache(fio, ...) sets caching behaviour
%
%  Cache:
%   readCache(fio, ...) reads item from cache if it exists
%   writeCache(fio, ...) writes to cache 
%   lsCache(fio, ...) lists current cache entrees
%   clearCache(fio, ...)  clear selected cache entrees
%  
%  File IO for images is done by imFileRead while for other files
%  (based on extenstion) by specialized functions whose name are
%  of the form extFileRead. The structure params is passed into
%  each function. To add another file type, just create a reader function
%  with the last 3 leters of the extension somewhere on the matlab path. 
%
%  Simple example:
%    fio = frameIo('myDir/mySubDir', 'im_*.jpg', {[1 3 5 7 9], [11 4 7]});
%        % make frame io object with all files 'myDir/mySubDir/foo_*pgm' and 2 defined subseqs
%    fio = setIm(fio, 1); % sets subsequence to [1 3 5 7 9]
%    n = length(fio); % length is the length of the current subsequence
%    for f=1:length(fio) %show all images in current subseqence
%      subplot(1,n,f); imagegr(fio{f});
%    end 
  
  
  if (nargin==1)
    % if single argument of class frameIo, return it
    if isa(dirpath,'frameIo')
      fio = dirpath;
      return
    end 
  end
  
  %make defult values
  if(exist('dirpath')~=1)  dirpath = './';  end
  if(exist('pattern')~=1)  pattern = 'im_*';  end
  if(exist('specimname')~=1)  specimname = {};  end
  if(exist('subseqs')~=1)  subseqs = {};  end
  if(exist('params')~=1)  params = struct('rot',0,'rescale',0);  end
  if(dirpath(end)=='/')
    dirpath=dirpath(1:end-1);
  end
  prefix ='';
  stream = 0;
  
  if(~iscell(pattern))
     a = [findstr(pattern, '*') findstr(pattern, '?')];
     if(length(a)>1) 
       prefix = pattern(1:min(a));
     end
     pattern = {pattern};
  end
  
  if(strcmp(pattern{1},'avi'))
    b=0;
    [c,b] = getAviFrames(dirpath, 0, 0);
    b
    if(b<=0)
      error(['AVI File not found or empty ' dirpath]);
    end
    dirlist = cell(1,b);
    for i=1:b  % ugly - is there a better way?
      dirlist{i} = i-1;
    end
    stream = 1;
  else
    dirlist = dir([dirpath '/' pattern{1}]);
    for i=2:length(pattern)
      dirlist = cat(1, dirlist, dir([dirpath '/' pattern{i}]))
    end

    dirlist = sort({dirlist.name});
    if(length(dirlist)==0)
      error(['No files found: ' dirpath '/' pattern{1}]);
    end
  end
  
  try
    cacheDir = findCacheDir(dirpath);
    useCache = 3;
  catch 
    cacheDir = '/no_cache_dir/try_setting_it';
    useCache = 0;
  end

  fio = struct('dirpath', dirpath, 'pattern', {pattern}, 'prefix', prefix, ...
               'cacheDir', cacheDir, 'params', params, 'curInd', 0, ...
	       'dirlist', {dirlist}, 'stream', stream, 'specim', {{}}, ...
	       'specimname', {specimname}, 'subseqs', {subseqs}, ...
	       'subseqind', zeros(length(subseqs),1), ...
	       'curseq', 0, 'useCache', useCache);
    
  for i=1:length(specimname)
    fio.specim{i} = double(mobilRead([dirpath '/' specimname{i}]));
  end
  
  fio = class(fio, 'frameIo');


  
  
  
