function [h,m,v,ind, histe,cent, bv] = binMoments(X, y, histe)
% create K-dim histogram and compute moments for each bin
%
%  [h,m,v, cent, bv] = binMoments(X, y, histe)
%
%  where X (n-by-k) is the coordinates of each datapoint
%  y (n-by-j) the value of the datapoint  
%    if y is [], outputs m, v = 0
%    if y has multiple columns, m{j} & v{j} contain the mean and
%    var of each column.
%  histe{i} for 1<i<k is are the edges of the bins or an integer specifying the
%           number of bins. If a number (m), then m bins are used along that 
%           dimension spaced so that each bin has approximately the same number of 
%           samples.
%
% outputs:
%  h (k-dim) is the histogramm (number of samples that fell into that bin)
%  m (k-dim) is the mean of the y values of the points in the bin
%  v (k-dim) is the var of the y values of the points in the bin
%    if size(y,2)>1, then m{j}, v{j} is the mean,var of each column
%
%  ind (1-n) is the index into the histogram of each datatapoint (X(i,:), y(i))
%  cent{i} (1-m), is the center of each bin along dimension i
%  bv (k-dim) is the volume of each bin (for density estimates) 
%
%  see histc2 for a 2-d version of this with output h...

  assert(size(X,2)==length(histe), 'length of histe must equal the number of columns of X'); 
  nbin = [];
  
  for i=1:size(X,2)
    x=X(:,i);
    if(length(histe{i})==1)
      rnbin = histe{i};
      cnbin=rnbin;
      [sx] = sort(x);
      %while(length(histe{i})<=rnbin)
      ind = round(linspace(1,length(x),2*cnbin+1));
      histe{i} = unique(sx(ind(1:2:end)));
      %cnbin=cnbin+1;
      %end
      histe{i}(end)= histe{i}(end)+abs(histe{i}(end)).*[eps];
      cent{i} = sx(ind(2:2:end));
    else    
      histe{i} = reshape(histe{i}, [prod(size(histe{i})) 1]);
      cent{i} = (histe{i}(2:end)+histe{i}(1:end-1))/2;
    end
    
    nbin(1,i) = length(histe{i})-1;
    [h,bin(:,i)] = histc(x, histe{i});
    
    len{i} = (histe{i}(2:end)-histe{i}(1:end-1));
  end
  % get rid of points that are out of range...
  good = find(sum(bin==0 | bin>repmat(nbin, [size(bin,1),1]), 2)==0);
  for i=1:size(X,2)
    cbin{i} = bin(good,i);
  end
  nbin = [nbin 1];
  
  %make a uniqe number for each bin:
  ind = sub2ind(nbin, cbin{:}); 
  %create hist/moment matrix 
  h = zeros(nbin);
  ysize = size(y);
  m={}; v={};
  for j=1:ysize(2)
    m{j} = zeros(nbin);
    v{j} = zeros(nbin);
  end
  
  for i=1:max(ind)
    h(i) = sum(ind==i);
    if(h(i)>0 & ysize(2)) 
      val = y(good(ind==i), :);
      for j=1:ysize(2)
	m{j}(i) = mean(val(:,j));
	v{j}(i) = var(val(:,j));
      end
    end
  end

  if(ysize(2)==0)
    m=[]; v=[];
  elseif(ysize(2)==1)
    m=m{1}; v=v{1};
  end
  
  if(nargout>3)
    ind2 = zeros(length(y),1);
    ind2(good) = ind;
    ind = ind2;
  end
  
  % compute bin  volume as 6th output
  if(nargout>6)
    bv = ones(nbin);
    for i=1:size(X,2)
      bv=bv.*repiperm(len{i}, i, size(bv));
    end
  end
  
      
  
      
