function [h, ba, cent, histel] = histc2(x, y, histex, histey, fignum)
% create 2-D histogram image 
%
%  [h, ba, cent] = histc2(x, y, histex, histey, fignum)
%
%  where x,y are the data values along the x, y axis
%  and histex, histey define the edges of the bins of the histogram.
%  Resulting hist image h will be (histey-1) x (histex-1).
%
%  fignum (o) - set the display number of the figure. 
%  if 0 or not given but output arguments are to be returned, then nothing is 
%  displayed. if -1, it is displayed in the current figure.
%
%  optional output arguments
%  h    the 2-D histogram
%  ba   is an image where each entree is the area of each bin (for density estimates)
%  cent is the coordinate of the center of each bin within histex and histey 
%        space 
%
%  hint: see also histc2moments such as
%    [h, ba,cent] = histc2(x, y, 10, 20, 1); 
%    histc2moments(h, cent);
%
%  to show the results outside of this function:
%    cx = cent(1,:,2); cy = cent(:,1,2);
%    imagesc(cx,cy, h); axis xy; xlabel('X data'); ylabel('Y data');

%
  if(nargout==0 | (nargin>4 & fignum))
    showit=1;
  else
    showit=0;
  end
  
  y = makecol(y);
  x = makecol(x);
  
  if(length(histex)==1)
    [sx] = sort(x);
    sx(end)=sx(end)+eps; % make sure last value is in it...
    ind = round(linspace(1,length(x),2*histex+1));
    histex = sx(ind(1:2:end));
    cx = sx(ind(2:2:end));
  else    
    histex = reshape(histex, [prod(size(histex)) 1]);
  end
  
  if(length(histey)==1)
    [sy] = sort(y);
    sy(end)=sy(end)+eps; % make sure last value is in it...
    ind = round(linspace(1,length(y),2*histey+1));
    histey = sy(ind(1:2:end));
    cy = sy(ind(2:2:end));
  else    
    histey = reshape(histey, [prod(size(histey)) 1]);
  end
  
  [h1,bin] = histc(x, histex);
  
  for i=1:length(histex)-1
    if(any(bin(:)==i))
      h(:, i) = histc(y(bin==i)', histey)';
      if(nargout>3)
	histel{i}=y(bin==i);
      end
      %h(:,i) = h(:,i)./sum(bin(:)==i);
    else
      h(1:length(histey),i) = 0;
      histel{i}=[];
    end
  end
  h = h(1:end-1,:);
  
  % compute bin areas as 2nd output
  if(nargout>1)
    ba = (histey(2:end)-histey(1:end-1))*(histex(2:end)-histex(1:end-1))';
  end
  
  %compute center of each output histogram pixel as 3rd output
  if(nargout>2 | showit)
    if(exist('cx')~=1)
      cx = (histex(2:end)+histex(1:end-1))/2;
    end
    if(exist('cy')~=1)
      cy = (histey(2:end)+histey(1:end-1))/2;
    end
    [Y, X] = ndgrid(cy, cx); 
    cent = cat(3,Y,X);
  end

  if(showit)
    if(nargin>4 & fignum>0)
      figure(fignum); 
    end
    imagesc(cx,cy, h);
    axis xy;
    xlabel('X data');
    ylabel('Y data');
  end
      
      
