//
//
// mex interface to avifile libraries
//
// charless fowlkes <fowlkes@eecs.berkeley.edu>
// 8.21.2003
//
//

#include <mex.h>
#include <math.h>
#include <string>
#include <fstream>
#include <iostream>
#include <vector>
#include <stdio.h>
#include <stdlib.h>
#include "avifile.h"
#include "aviplay.h"
#include "version.h"
#include "utils.h"
#include "avm_output.h"


/*
 *  To mexify:    
 *

 g++ -c  -I/usr/local/include/avifile -I/usr/sww/pkg/matlab-6.5/extern/include -I/usr/sww/pkg/matlab-6.5/simulink/include -DMATLAB_MEX_FILE -fPIC -ansi -D_GNU_SOURCE -pthread  -O -DNDEBUG getAviFrames.cc

gcc -c  -I/usr/local/include/avifile -I/usr/sww/pkg/matlab-6.5/extern/include -I/usr/sww/pkg/matlab-6.5/simulink/include -DMATLAB_MEX_FILE -fPIC -ansi -D_GNU_SOURCE -pthread  -O -DNDEBUG /usr/sww/pkg/matlab-6.5/extern/src/mexversion.c

 g++ -O -pthread -shared -Wl,--version-script,/usr/sww/pkg/matlab-6.5/extern/lib/glnx86/mexFunction.map -o getAviFrames.mexglx  getAviFrames.o mexversion.o  -laviplay -Wl,--rpath-link,/usr/sww/pkg/matlab-6.5/extern/lib/glnx86,--rpath-link,/usr/sww/pkg/matlab-6.5/bin/glnx86,--rpath,/usr/local/lib/avifile0.7,--rpath,/usr/lib/win32,--rpath,/usr/local/lib/ -L/usr/sww/pkg/matlab-6.5/bin/glnx86 -lmx -lmex -lmat -lm -lm 

 *
 *
 */

using namespace std;

struct AviFileCache {
  AviFileCache(char * infilename) {
    inVidStr = 0; inFile=0; lastFrame=-100;    
    filename = new char [strlen(infilename)+2];
    strcpy(filename, infilename);
    inFile = avm::CreateReadFile(filename);

    if (!inFile || !inFile->IsValid()) 
      {
	char msg[512];
	sprintf(msg, "Error opening %s.  Aborting",filename);
	mexErrMsgTxt(msg);
	return;
      }
    inVidStr = inFile->GetStream(0, avm::IStream::Video);
    inVidStr->StartStreaming();
  }

  char* filename;   
  avm::IReadFile* inFile;
  avm::IReadStream* inVidStr;
  int lastFrame;
};



#define DBG_LEVEL -1

void 
mexFunction (
    int nlhs, mxArray* plhs[],
    int nrhs, const mxArray* prhs[])
{
    if (nlhs < 0) {
        mexErrMsgTxt("Too few output arguments.");
        return;
    }
    if (nlhs > 2) {
        mexErrMsgTxt("Too many output arguments.");
        return;
    }
    if (nrhs < 2) {
        mexErrMsgTxt("Too few input arguments.");
        return;
    }
    if (nrhs > 4) {
        mexErrMsgTxt("Too many input arguments.");
        return;
    }
    
    avm::out.resetDebugLevels(DBG_LEVEL);
    //get the filename
    int buflen = (mxGetM(prhs[0]) * mxGetN(prhs[0])) + 1;
    char* filename = (char*)mxCalloc(buflen, sizeof(char));
    if (mxGetString(prhs[0], filename, buflen) != 0)
    {
      mexErrMsgTxt("Problem groking filename.");
      return;
    }

    // check avifile library version
    if (GetAvifileVersion() != AVIFILE_VERSION) {
       char msg[512];
       sprintf(msg, "This binary was compiled for Avifile ver. %d, , but the library is ver. %d.\nAborting\n",
                AVIFILE_VERSION,GetAvifileVersion());
       mexErrMsgTxt(msg);
       return;
    }


    static vector<AviFileCache>* fcachePtr=0;
    if(fcachePtr==0) {
      fcachePtr = new vector<AviFileCache>();
    }
    vector<AviFileCache>& fcache = *fcachePtr;

    vector<AviFileCache>::iterator fiter = fcache.begin(); 
    for(; fiter!=fcache.end();  fiter++) {
      if(strcmp(filename, fiter->filename)==0) {
	break;
      }
    }

    //and the frame numbers
    const int first = (int) rint(mxGetScalar(prhs[1]));
    int last = first;
    if(nrhs>2)
      last = (int) rint(mxGetScalar(prhs[2]));

    if(first==-1) {
      if(fiter!=fcache.end())
	fcache.erase(fiter);
      return;
    }

    //create cell array to return frames in
    if ((last-first) < 0)
    {
      mexErrMsgTxt("Last frame must occur after first frame.");
      return;
    }
    
    if(fiter==fcache.end()) {
      //open avi file
      AviFileCache newfc(filename);
      if(newfc.inVidStr==0)
	return;
      fiter = fcache.insert(fcache.begin(), newfc);
      if(DBG_LEVEL>0) cerr << "Made New." << endl;
    } else {
      if(DBG_LEVEL>0) cerr << "Found Old " << fiter->filename << endl;
    }
 
    
    avm::IReadFile* inFile = fiter->inFile;;
    avm::IReadStream* inVidStr = fiter->inVidStr;



    if(DBG_LEVEL>0)
      cerr << "Stream " << inVidStr->GetStreamInfo()->GetString() << endl;
	
    if(inVidStr->GetStreamInfo()->GetStreamFrames()<last) {
      char msg[512];
      sprintf(msg, "Last Frame Number %d beyond end of file %s.  Aborting\n",
	      last, filename);
      mexErrMsgTxt(msg);
      return;
    }

    if(plhs==0)
      return;

    int cdims[2];
    cdims[0] = (last-first)+1;
    cdims[1] = 1;
    plhs[0] = mxCreateCellArray(2,cdims);


    if(nlhs==2) {
      plhs[1] = mxCreateDoubleMatrix(1, 1, mxREAL);
      double  *x = mxGetPr(plhs[1]);
      x[0] = inVidStr->GetStreamInfo()->GetStreamFrames();
    }

    if(fiter->lastFrame!=first-1) {//decode starting from most recent keyframe 
      framepos_t kframe = inVidStr->GetPrevKeyFrame(first);
      for (framepos_t frame = kframe; frame < first; frame++)
	{
	  inVidStr->Seek(frame);
	  //cerr << "\rseeking " << inVidStr->GetPos();
	  inVidStr->ReadFrame(true);
	}
    } else {
      if(DBG_LEVEL>0) cerr << " Don't need to seek to " << first << endl;
    }
    inVidStr->Seek(first);

    //we've now the first of the desired frames
    //cerr << "\nfound first frame " << inVidStr->GetPos() << endl;
    avm::CImage *image1 = inVidStr->GetFrame(true);
    BitmapInfo  bmi(image1->GetFmt());
    bmi.SetRGB();
    bmi.SetBits(24);
    avm::CImage *image = new  avm::CImage(image1, &bmi);
    image1->Release();

    if(DBG_LEVEL>0)
      cerr << "Image " << avm_img_format_name(image->Format()) << " " << image->Bpp() << " " << image->Bpl() 
	   << " " << image->Depth() << " " << image->Width() 
	   << " " << image->Height() << " " << image->GetFmt()->IsRGB() << endl ;
    
    //get frame dimensions
    int dims[3];
    dims[0] = image->Height();
    dims[1] = image->Width();
    dims[2] = 3;  //3 colorplanes

    //copy into matlab matrix
    mxArray* framedata = mxCreateNumericArray(3,dims,mxUINT8_CLASS,mxREAL);
    uint8_t* imbuf = (uint8_t*)mxGetData(framedata);
    for (int i = 0; i < dims[0]; i++)
    {
      for (int j = 0; j < dims[1]; j++)
      {
         uint8_t* base = image->At(j,i);
         imbuf[i + j*dims[0] + 0*(dims[0]*dims[1])] = *(base+2);
         imbuf[i + j*dims[0] + 1*(dims[0]*dims[1])] = *(base+1);
         imbuf[i + j*dims[0] + 2*(dims[0]*dims[1])] = *(base+0);
      }
    }
    mxSetCell(plhs[0],0,framedata);   //put in cell array 
    image->Release();                 //free image
    //and the remaining frames
    for (framepos_t frame = first+1; frame <= last; frame++)
    {
      inVidStr->Seek(frame);
      //cerr << "\rripping " << inVidStr->GetPos();
      image1 = inVidStr->GetFrame(true);
      image = new  avm::CImage(image1, &bmi);
      image1->Release();

      framedata = mxCreateNumericArray(3,dims,mxUINT8_CLASS,mxREAL);
      uint8_t* imbuf = (uint8_t*)mxGetData(framedata);
      for (int i = 0; i < dims[0]; i++)
      {
        for (int j = 0; j < dims[1]; j++)
        {
           uint8_t* base = image->At(j,i);
           imbuf[i + j*dims[0] + 0*(dims[0]*dims[1])] = *(base+2);
           imbuf[i + j*dims[0] + 1*(dims[0]*dims[1])] = *(base+1);
           imbuf[i + j*dims[0] + 2*(dims[0]*dims[1])] = *(base+0);
        }
      }
      mxSetCell(plhs[0],frame-first,framedata);
      mxSetCell(plhs[0],frame-first,framedata);
      image->Release();
    }
    fiter->lastFrame = last;

    //cerr << "\nfound last frame " << inVidStr->GetPos()-1 << endl;
}


