function params = makeParams(varargin)
% makeParams   utility for making paramter structures where defaults can be overwritten 
%
% params = makeParams(def_params..., user_params...)
% def_params... - default parameters.  structure with default values
% user_params... - user parameters that replace default parameters.
%   both of these may be one or more:
%        structure 
%   or
%        'name1', 'value1' pairs 
%                 
% Note: actually, there is no distinction between "def_params" and "user_params". The
%       argument list is actually treated as a sequence of <struct>'s, and <name,value>
%       pairs in any order. so 
%
%           makeParams(struct_a, 'name1', value1, struct_b, struct_c, 'name2', value2)
%
%       is valid, where the rule is that later field values with the same filed name 
%       replace earlier ones (so if 'name2'=='name1', then params.name1=value2). 
%
% Example use in a initParams function for some library that
%   uses parameters 'foo' and 'bar' from struct params:
% 
%  function params = initParams(varargin);  
%    % varargin may be <'name',value> pairs, or a single structure
%    
%    % set defaults
%    params = struct('foo', 1, 'bar', [1 2 3 4]);
%    % add user values
%    params = makeParams(params, varargin{:});
%
%    % alternately (produces same result)
%    params = makeParams('foo', 1, 'bar', [1 2 3 4], varargin{:});
%

  % if we get no arguments in, just return blank struct
  if(nargin==0)
    params.none=[];
    return;
  end

  % if first arg is a struct, just set params to be it
  if(isstruct(varargin{1}))
    params = varargin{1};
    s=2; i=0;
  else
    params = struct('');
    s=1; i=0;
  end
  
  % for each of the remaining arguments, do:
  %   if arg "i" is struct. If so, add its fields to params (replacing conflicts). 
  %   if arg "i" is a string "a", make a field in parmas.a=<arg i+1>
  
  i=s;
  while(i<=nargin)
    if(isstr(varargin{i}) & i<nargin)
      if(isstruct(varargin{i+1}) & isfield(params,varargin{i}) & ...
	isstruct(getfield(params, varargin{i})))
	params = setfield(params, {1,1}, varargin{i}, ...
			mergeStructs(getfield(params, varargin{i}), ...
				     varargin{i+1}));
      else
	params = setfield(params,{1,1},varargin{i},varargin{i+1});
      end
      i=i+2;      
    elseif(isstruct(varargin{i}))
      params = mergeStructs(params, varargin{i}); 
      i=i+1;
    else
      error('Could not parse arguments.');
    end
  end

  