function betas=lars(X,y,Nsteps) 
% LARS Least Angle Regression 
% lars(X,y) returns the coefficients at each step in LARS regression 
% INPUT: X is the design matrix of n by m, 
%          where n is the number of examples and m is the number of features 
%        y is the response vector of n by 1 
% OUTPUT:
%        betas contains the regression coefficient at each step as 
%          columns. It is a m by m matrix.
% 
% Written by Hao Zhang.
%
% Reference: Bradley Efron et al, "Least Angle Regression", source
% from http://www-stat.stanford.edu/~hastie/pub.htm

  eps_tol=1E-7;
  assert(norm(sum(X,1))<eps_tol, 'zero mean covariates')
  assert(norm(sum(X.^2,1)-1)<eps_tol, 'covariates must have norm 1');
  assert(norm(sum(y))<eps_tol, 'zero mean response');
  
  [n,m]=size(X);
  mu_hat=zeros(n,1); % the current response on a subset of features
  beta=zeros(m,1); % the initial beta is all zero

  if(cond(X)>1e8)
    disp('bad cond');
    X = X.*(1+rand(size(X))*0.001);
  end
  
  y_proj=X*(X\y); % y projected onto the space spanned by the covariates
  for k=1:Nsteps
    fprintf('%d.',k);
    C=X'*(y-mu_hat); % C is the current correlation
    C_hat=max(abs(C));  % the max of the correlations
    Acurl=find(abs(abs(C)-C_hat)<eps_tol); % the set of covariates
                                           % that are  maximally
                                           % correlated
    if(length(Acurl)~=k)
      return;
    end
    assert(length(Acurl)==k, 'k covariates at step k'); % ensure a
                                                        % new
                                                        % covariate
                                                        % is added
                                                        % every
                                                        % time
    s=sign(C(Acurl)); % get the sign of the maximal correlations
    XA=X(:,Acurl).*repmat(s',n,1); % the columns of XA is
                                   % sign-corrected covariates,
                                   % ready for equal angular
                                   % computation
    beta_inc=(XA'*XA)^(-1)*ones(k,1);  % the increment in beta,
                                       % derived in correspondence
                                       % with formula for 'u' as
                                       % below
    u=pinv(XA')*ones(k,1); % the vector that makes equal angle to
                           % the columns of XA, note that it is
                           % DIFFERENT than the mean of the
                           % columns! (they are the same only for 2
                           % dimensional vectors) Also, in fact u
                           % does not need to be of unit length as
                           % in the paper because its norm is
                           % canceled in later computations
    
    
    a=X'*u;  % the vector of inner products
    if (k==m) % the last step needs special treatment: hit the projected y
      finalgamma=(y_proj-mu_hat)./u;
      gamma=mean(finalgamma);
      assert(std(finalgamma)<eps_tol); % all should advance towards
                                       % the final, projected y
    else
      Acurl_c=setdiff([1:m], Acurl);  % set of non-maximally
                                      % correlated covariates
      C=C(Acurl_c); a=a(Acurl_c); % focus on the set of
                                  % non-maximally correlated
                                  % covariates
      gammas=[(C_hat-C)./(1-a); (C_hat+C)./(1+a)]; % compute the
                                                   % values where a
                                                   % new covariate
                                                   % would join the
                                                   % set of
                                                   % maximally
                                                   % correlated
                                                   % covariates
      gammas=gammas(gammas>eps_tol); % take only positive gammas
      gamma=min(gammas); % move to the soonest event
    end
    mu_hat=mu_hat+gamma*u; % update the current response
    beta(Acurl)=beta(Acurl)+gamma*s.*beta_inc; % update beta
    
    betas(:,k)=beta; % record beta
  end
  
  %assert(norm(betas(:,end)-X\y)<eps_tol); % make sure the last beta
                                          % is the Ordinary Least
                                          % Squares solution
