function [C, ofs] = xcorr2(Template,X,submean,shape)
% xcorr2   normalized cross correlation similar to normxcorr2
% 
% [C, ofs] = normxcorr2Valid(Template,X, submean, 'shape')
%
% Template - n-channel template image
% X - n-channel image (larger or equal to template)
% submean - (o) if 1, uses the formula (x-xm)(y-ym)/(norm(x-xm)*norm(y-ym))
%               else if 0, no xm,ym are subtracted
%               (default = 0)
% shape - see filter2 (default = 'valid') 
%                  
% similar to normxcorr2 but:
%
% - returns the region specified by shape - see filter2
% - optionally no subtraction of the mean is performed
% - allows Template and X to be multi-channel (must have the 
%   same number of channels)
% 
  
  if(nargin<3)
    submean = 1;
  end
  if(nargin<4)
    shape = 'valid';
  end

  assert(all(size1(Template,[1 2 3]) <= size1(X,[1 2 3])), ...
	 'xcorr2: Template must be smaller or equal in size to X.');
  
  Template = double(Template);
  X = double(X);
  
  n = prod(size(Template));
  
  if(submean)
    Template = Template-mean(Template(:));
    Xs=0;
  end
  
  Y=0; Xn=0; 
  for i=1:size(X,3)
    Y = Y + filter2(Template(:,:,i),X(:,:,i), shape);
    Xn = Xn + conv2(X(:,:,i).^2, ones(size1(Template,[1 2])), shape);
    if(submean)
      Xs = Xs + conv2(X(:,:,i),ones(size1(Template,[1 2])), shape);
    end
  end
  
  % now Xs, Y, Xn are all 2-D mat the size specified by 'shape'
  
  if(submean) 
    % compute sum( (X-Xmean)^2 ) given sum(X^2), and X by
    % X^2 - 2*Xmean*X + Xmean^2 
    %   =>  X^2 - 2*n*Xmean + Xmean^2
    %   => sum(X^2) - (n * Xmean^2)
    Xn = Xn - (Xs.^2)/n;
    
    % compute (X-Xmean)*T, given X*T, Xmean, T
    Y = Y - Xs*mean(Template(:));
  end

  Tn = sum(Template(:).^2);
  C = Y./(sqrt(Tn.*Xn)+eps);

  ofs = ceil( (size1(X,[1 2]) - size1(Template,[1 2])+1 - size(C))/2);