function [hfraw] = hf_compRaw(pyr, p, para)
% hf_compRaw  extract the raw hyper-feature for each patch from pyr
%
% hfraw = hf_compRaw(pyr, p, para)
%
% pyr - model (a.k.a. left image) pyramid)
% p - the patches - n x 5: [scale, pos, size]
% para - parameters
% 
% hfraw - 
% names - cell array of string of length #raw_hyper_features

  sz = size1(pyr{1}.im,[1 2]);
  imrect = [1 sz(2) 1 sz(1)];
  % the object center in (x,y)
  center = sz([2 1])/2;
  
  % -------------------------------
  % global (whole object) features
  
  % compute median intensity of whole object center... (should consider using
  % mean here to allow fast computation)
  mrect = round( para.hf_mI_rect_frac .* ( sz([2 2 1 1])-1 ) )+1;
  mI = double(rgb2gray(pyr{1}.im(mrect(3):mrect(4), mrect(1):mrect(2), :)));
  % Global median Intensity feature
  gf.Ig = median(mI(:));
  
  % ----------------
  % for each scale:
  
  scales = unique(p(:,1));
  
  c=0;
  
  for si = 1:length(scales)
    lev = findClosestPyrLevel(pyr,scales(si));
    pis = find(p(:,1)==scales(si));
    
    % use double images 
    fim = double(pyr{lev}.fim)/(128*4);
    im = im2double(pyr{lev}.im);
    if(size(im,3)==3)
      im=rgb2gray(im);
    end
   
    %for each patch at this scale
    for pi=1:length(pis)
      c=c+1;
      pfim = id_cropPatch(fim, p(pis(pi),:), pyr{lev}.scale);
      pim = id_cropPatch(im, p(pis(pi),:), pyr{lev}.scale);

      % Position features: Scale, X, Y, Width, Height 
      pf = struct('Ps',p(pis(pi),1), ...
		  'Px', p(pis(pi),2)-center(1), ...
		  'Py', p(pis(pi),3)-center(2), ...
		  'Pw', p(pis(pi),4), ...
		  'Ph', p(pis(pi),5));
      
      % directional features (5)
      df = hf_directionalFeatures(pfim);
      
      % intensity features ()
      intf = hf_intensityFeatures(pim);
      
      % all of them 
      hfraw(c,1) = mergeStructs(mergeStructs(mergeStructs(pf, df), intf), gf);
    end   
  end
  
  % now convert array of struct with scalar fields to a single struct with array
  % fields
  hfraw = structArray2Struct(hfraw, 1);


  
function [df] = hf_directionalFeatures(pfim)  
% hf_directionalFeatures  computes directional features: vert, horiz, diag
%
% df = hf_directionalFeatures(pfim)  
% 
% pfim - filtered patch image with 4 directional channels at 45deg cw (v d1 h d2)
% df - Directional features: 'Dv', 'Dh', 'Df', 'Db', 'De'
%   Vertical, Horizontal, Forward diagonal ("/"), 
%   Backward diagonal ("\"), mean Energy  
%
% for each direction d (and its opposite o) this does 
%  f = sqrt( mean( max(0, d-o)^2 ) );
% while e is:
%  e = sqrt( mean( pfim^2) );
  
  % we add eps, so none will be exactly 0
  df.Dv  = sqrt(mean(mean(max(0,abs(pfim(:,:,1))-abs(pfim(:,:,3))).^2)))+eps;
  df.Dh  = sqrt(mean(mean(max(0,abs(pfim(:,:,3))-abs(pfim(:,:,1))).^2)))+eps;
  df.Df = sqrt(mean(mean(max(0,abs(pfim(:,:,2))-abs(pfim(:,:,4))).^2)))+eps;
  df.Db = sqrt(mean(mean(max(0,abs(pfim(:,:,4))-abs(pfim(:,:,2))).^2)))+eps;
  
  df.De = sqrt(mean(pfim(:).^2))+eps;

function [intf] = hf_intensityFeatures(pim)  
% hf_intensityFeatures  computes intensity features: vert, horiz, diag
%
% intf = hf_intensityFeatures(pfim)  
% 
% pim - rgb or gray patch image 
% intf - Intensity features: 'Im', 'Iv', 'In', 'Ix', 'Iw', 'Ib'
%        Median, Variance, miN, maX, fraction White, fraction Black
%

  intf.Im = median(pim(:));
  intf.Iv = var(pim(:));
  intf.In = min(pim(:));
  intf.Ix = max(pim(:));
  intf.Iw = mean(pim(:)>0.95);
  intf.Ib = mean(pim(:)<0.05);
