function cid = h_generateCid(hid, mim, varargin)
% h_generateCid   generate an identifier given a single model image 
% 
% cid = h_generateCid(hid, mim, params...)
%
% hid - a category-trained identifier generator model (see t_learnHid.m)
% mim - model image (as image or prep'ed pyramid)
% params... - parameters to change (replaces hid.para fields)
%           passed in as a struct and/or as 'name',value pairs
%           see makeParams.
%
% steps:
%    prep model image
%    define candidate patches
%    for each candidate patch:
%        extract hyper-features 
%        compute same/diff distributions
%        compute mutual information 
%    pick cascade order
 
  
  % update parameters 
  para = makeParams(hid.para, varargin{:});
  cid.para = para;
  
  % if mim is not a pyramid already, make it
  cid.pyr = id_prepImCache(mim, '', para);

  % define candidate patches
  cid.p = id_pickCandiatePatches(cid.pyr, para);
  
  %--------------------
  %for each patch:
  
  % compute hyper features
  cid.hf = hf_compAll(cid.pyr, cid.p, hid.hfdat, para);
  
  % estimate same & different gamma distributions
  cid.D = h_estDistributionsFromHF(cid.hf, hid, para);
  
  % compute mutual information score (for each patch independently)  
  cid.mi = mi_compMutInfoFromD(cid.D, para);
  
  % ------------------
  % cache p and hf.base with pyramid (id_pickCandiatePatches and
  % hf_compBase look for these)
  cid.pyr{1}.pat.p = cid.p;
  cid.pyr{1}.pat.hfbase = cid.hf.base;
  cid.pyr{1}.pat.hfbase_tag = para.hfbase_tag;
  id_writeCache(cid.pyr, cid.pyr{1}.name, para);
  
  % -------------------
  % pick patch order

  % an optimization (pairflat is deleted below)
  C = struct2cell(cid.hf.pair); cid.hf.pairflat = cat(2, C{:});
  ord=[];
  cmi=cid.mi;  
  for i=1:min(para.max_patches_in_c, length(cid.mi))
    [ord(i,:), cmi] = h_findNextPatch(ord, cmi, cid, hid, para);
    
    % if the conditional mutual information of this patch is less then
    % para.min_info, then stop.
    if(ord(i,3) < para.min_info)
      ord = ord(1:i-1,:);
      break;
    end
  end
  
  cid.ord = ord;
  
  % copy stopping thresholds from hid
  cid.thresh = hid.thresh;
  
  if(para.dbg>1) % if we are debugging
    % add hid model 
    cid.hid = hid;
    cid.hf.pairflat =[];
  else % no debug
    %clear hyper-features (no longer needed)
    cid.hf = [];
  end

  if(para.show_steps)
    mdisp('Created Cascade with ', size(cid.ord,1), ...
	  ' patches. CMI range: ', cid.ord(1,3), cid.ord(end,3)); 
  end
  
