function [alpha, muim, vim] = t_fitGLM(x,y, para)
% t_fitGLM   fit Gamma parameters to the y data conditioned on x
%
% [alpha, muim, vim] = t_fitGLM(x,y, para)
%
% alpha.{mu,v} - records the fitted GLM parameters for the gamma dist
% muim, vim  - for debugging - records the steps of the optimization
  
  ;
  % set up link functions 
  mu_func = @muFuncWIdentityLink;
  v_func = @vFuncWIdentityLink;
  vmu_func = @vmuFuncIdId;
  
  % olm estimate of mu
  alpha.mu = x\y;  
  mu = x*alpha.mu;
 
  % estimate v also by a linear model of abs(y-mu)
  err = abs(y-mu);
  s = x\err;
  sig = x*s;
  v = mu.^2./sig.^2;
  alpha.v = x\v;
  % alpha.v(1)=1;
  
  %v = ones(length(y),1);

  muim = repmat(alpha.mu, [1 para.num_fit_iter+2]);
  vim = repmat(alpha.v, [1 para.num_fit_iter+2]);

  nlls = 50000;
  [nlls,muterm] = feval(mu_func, alpha.mu, x, y, v);
  [nlls] = feval(v_func, alpha.v, x, y, muterm);

  if(para.show_glm_opt>0)
      mdisp('Start: nlls = ', nlls);
      mdisp('  mu: ', alpha.mu')
      mdisp('  v: ', alpha.v')
  end
  
  % now we optimze the GLM using very inefficient fminsearch.
  % We iterate between optimzing mu and v, increasing the number of 
  % samples as we go
  % TODO: use convex optimization to do this faster
  for i=1:para.num_fit_iter
    opt = optimset('Display', 'off', 'TolX', 0.05, 'TolFun', ...
		     max(50000,abs(nlls))/(50000*4), 'MaxIter', 1000);
    
    maxelm = min(40000,5e3*4^(i-1));
    if(length(y)>maxelm)
      subsample = randperm(length(y));
      subsample = subsample(1:maxelm);
    else
      subsample = 1:length(y);
    end
    
    % we first optimize mu
    alpha.mu = fminsearch(mu_func, alpha.mu, opt, x(subsample,:), y(subsample), v(subsample));
    [nlls,muterm] = feval(mu_func, alpha.mu, x, y, v);
    
    % then v 
    alpha.v = fminsearch(v_func, alpha.v, opt, x(subsample,:), y(subsample), muterm(subsample));     
    [nlls,v] = feval(v_func, alpha.v, x, y, muterm);
    
    %update debug images
    muim(:,i+1) = alpha.mu;
    vim(:,i+1) = alpha.v;
    
    if(para.show_glm_opt>0)

      mdisp('Iter ', i, ': nlls ', nlls, ' samples ', length(subsample));
      mdisp('  mu: ', alpha.mu')
      mdisp('  v: ', alpha.v')
      mdisp('  Opt Param: ', opt.TolX, opt.TolFun, opt.MaxIter, 'st ', nlls);      
    end
  end
  
  
function [nlls, muterm, mu] = muFuncWinvLink(alpha_mu, x, y, v)
%function to fit alpha_mu to gamma log like with link function: link = 1/mu;
  
  link = max(1e-4,x*alpha_mu);
  muterm = (-y.*link + log(link));
  nlls = -sum(v.*muterm);
  mu=0;
  if(nargout>2)
    mu = 1./link;
  end
  
function [nlls, muterm, mu] = muFuncWIdentityLink(alpha_mu, x, y, v)
%function to fit alpha_mu to gamma log like with link function: link = mu;
  
  link = max(1e-4,x*alpha_mu);
  muterm = (-y./link - log(link));
  nlls = -sum(v.*muterm);
  mu=link;

function [nll, v] = vFuncWIdentityLink(alpha_v, x, y, muterm)
%function to fit alpha_v with gamma log like with v link function link = v = mu^2/var.
  
  link = max(1e-4,x*alpha_v);
  nll = -sum(link.*muterm + link.*log(y)+ link.*log(link) - gammaln(link));
  v=link;
  
function [nll, v, mu] = vmuFuncInvId(alpha, x, y)
%function to fit alpha_v with gamma log like with v link function link = v = mu^2/var.
  
  vterm = max(1e-4,x*alpha(size(x,2)+1:end));
  muterm = max(1e-4,x*alpha(1:size(x,2)));
  nll = -sum(vterm.*(-y.*muterm + log(muterm)) + vterm.*log(y) + vterm.*log(vterm) - gammaln(vterm));
  v = vterm;
  mu = 1./muterm;
  
function [nll, v, mu] = vmuFuncIdId(alpha, x, y)
%function to fit alpha_v with gamma log like with v link function link = v = mu^2/var.
  
  vterm = max(1e-4,x*alpha(size(x,2)+1:end));
  muterm = max(1e-4,x*alpha(1:size(x,2)));
  nll = -sum(vterm.*(-y./muterm - log(muterm)) + vterm.*log(y) + vterm.*log(vterm) - gammaln(vterm));
  v = vterm;
  mu = muterm;
