function [hid, tr] = t_trainHid(l_ims, s_ims, d_ims, varargin) 
% t_trainHid   off-line category training for the id generator H
%
% [hid, tr] = t_trainHid(l_ims, s_ims, d_ims, varargin) 
%
% l_ims - left images - nx1 cell array
% s_ims - right "same" images - n x m cell array
% d_ims - right "different" images - n x k cell array
%
% each cell array element above may be:
%    string - filename of image or .mat file
%    single image
%    cell array of images 
%    pyr structure
%    pyr structure with precomputed patches + hf's (pyr{1}.pat)
%
%
% steps: 
%   for each "left" image:
%      prep image (image -> pyramid)
%      prep corresponding same and diff images
%      define candidate patches on left pyramid
%      for each candidate patch:
%         extract base hyper-features (hfbase) 
%         match patch to same and diff images (da)      
%      record hfbase, da_same, da_diff
%   combine hfbase and da from all images 
%   do hyper-feature selection (hfbase -> hfselect) 
%   fit GLMs for [hfselect, da_same] and [hfselect, da_diff] 

  ;
  
  % init parameters
  para = id_initParams(varargin{:});
  hid.para = para; 
  
  % init training data struct
  tr.sm = [];
  tr.dm = [];
  
  % for each left image
  for li = 1:length(l_ims)
    clear sm dm;
    
    % load images & make pyramids
    [lpyr, spyr, dpyr] = loadImages(l_ims{li}, {s_ims{li,:}}, {d_ims{li,:}}, ...
				    li, para);
    
    % define candidate patches
    lpyr{1}.pat.p = id_pickCandiatePatches(lpyr, para);
    
    % for each patch, compute base hyper-features
    lpyr{1}.pat.hfbase = hf_compBase(lpyr, lpyr{1}.pat.p, para);
    
    %save these out into the cache
    lpyr{1}.pat.hfbase_tag = para.hfbase_tag;
    id_writeCache(lpyr, lpyr{1}.name, para);
    
    %match to same
    for i=1:length(spyr)
      sm(:,:,i) = t_matchAllPatches(lpyr, spyr{i}, lpyr{1}.pat.p, para);
    end
    
    %match to diff 
    for i=1:length(dpyr)
      dm(:,:,i) = t_matchAllPatches(lpyr, dpyr{i}, lpyr{1}.pat.p, para);
    end
    
    % concatinate vertically
    tr.sm = [tr.sm ; sm];
    tr.dm = [tr.dm ; dm];
    tr.hfbase(li) = lpyr{1}.pat.hfbase;
  end

  % change hfbase to flat representation where each field contains all of the
  % values from all left image patches 
  tr.hfbase = structArray2Struct(tr.hfbase, 1);
  
  %save training data into cache:
  tr.hfbase_tag = para.hfbase_tag;
  tr.match_tag = para.match_tag;
  
  % find normalization for the hyper-features
  [tr.hfnorm, tr.normal] = hf_normalize(tr.hfbase, [], para);
  hid.hfdat.normal = tr.normal;
  
  %select top N hyper-features
  [hfac, terms] = hf_compAllCandidates(tr.hfnorm, para);
  tr.selterms = hf_select(hfac, terms, mean(tr.sm(:,3,:),3), para);
  hid.hfdat.selterms = tr.selterms;
  clear hfac; clear terms;
  tr.s = hf_compSelect(tr.hfnorm, tr.selterms, para);

  % learn GLM models
  hid.sglm = t_trainGLM(tr.s, tr.sm(:,3,:), para, 0);
  hid.dglm = t_trainGLM(tr.s, tr.dm(:,3,:), para, 1);

  % do pairwise training
  [hid.hfdat.pairterms, hid.pairmodel] = ...
      t_trainPairs(tr.hfbase, tr.sm(:,3,:), tr.dm(:,3,:), para);

  % compute accept/reject stopping thresholds
  hid.thresh = t_trainStoppingThresh(hid, l_ims, s_ims, d_ims, para); 

function [lpyr, spyr, dpyr] = loadImages(l_im, s_im, d_im, li, para)
% loadImages   make pyr from left image and corresponding same and diff images
%
%  [lpyr, spyr, dpyr] = loadImages(l_im, s_im, d_im, li, para)

  lpyr = id_prepImCache(l_im, ['trLeft' num2str(li, '%02d')], para);
  
  for i=1:length(s_im)
    spyr{i} = id_prepImCache(s_im{i}, ['trSame' num2str(li, '%02d') '_'  num2str(i)], para);
  end

  for i=1:length(d_im)
    dpyr{i} = id_prepImCache(d_im{i}, ['trDiff' num2str(li, '%02d') '_'  num2str(i)], para);
  end


  