function [cim, offset] = id_cropPatch(im, p, imscale, margin)
% id_cropPatch   crop the area of a patch out of an image or pyramid
%
% [cim, offset] = id_cropPatch(im, p, imscale, margin)
%
% im - either an image at scale imscale or a whole pyramid
% p - 1x5 descriptor of a patch: [scale, posx, posy, radx, rady]
% imscale - (o) the scale of the image im (if not given, uses p(1))
% margin - (o) add a margin of this thickness in pix at scale 0 to the patch. 
%          margin may be: [all], [x y], or [l r t b] 
%          positive number expands the patch, negative contracts
%
% cim - croped image
% offset - offset of the croped image rect to the patch rect. 
%          e.g. offset = [-1 1 -1 1] means that cim is 1 pixel larger 
%          at this scale then the patch p in each of the 4 directions.

  
  if(nargin<3 | length(imscale)==0)
    imscale = p(1);
  end

  if(nargin<4)
    margin = [0 0 0 0];
  elseif(length(margin)==1)
    margin = margin([1 1 1 1]);
  elseif(length(margin)==2)
    margin = margin([1 1 2 2]);
  end

  % if we were given a pyramid
  if(iscell(im))
    lev = findClosestPyrLevel(im,p(1));
    imscale = im{lev}.scale;
    im = im{lev}.im;
  end
  
  % convert p and margin to th right scale
  pl = p(2:5)*(2^-imscale);
  margin = margin*(2^-imscale);
  
  sz = size(im);
  
  % the image rect
  imrect = [1 sz(2) 1 sz(1)];

  % the patch rect
  prect = round([pl(1)+[-pl(3) pl(3)] ...
		 pl(2)+[-pl(4) pl(4)]]);

  % the requested crop rect 
  irect = round([pl(1)+[-pl(3)-margin(1) pl(3)+margin(2)] ...
		 pl(2)+[-pl(4)-margin(3) pl(4)+margin(4)]]);
  
  % intersect irect with image rect
  irectint = [max(imrect(1), irect(1)) ...
	      min(imrect(2), irect(2)) ...
	      max(imrect(3), irect(3)) ...
	      min(imrect(4), irect(4))];
  
  % cut out irectint area
  cim = im(irectint(3):irectint(4),irectint(1):irectint(2),:);
  
  % compute offset
  offset = irectint-prect; 