function pyr = id_prepIm(im, para, varargin)
% id_prepIm   prepare an image: precompute gauss pyr, filter, etc.
%
% pyr = prepIm(im, para, varargin)
%
% im - can be: string (image or .mat filename), rgb im, gray im, 
%      or full pyr (in which case, it is just returned)
% 
% para.pyr_scales = [top_octave step_size] in octaves. e.g. [3 0.5]
%      will create images of size s, s/sqrt(2), s/2, ..., s/8]
%
% varargin - <'fieldname', value > pairs for adding information to 
%            the struct at every level of the pyramid
% special varargin: <'name', string> in addition to labeling each level 
%            with this name, this checks to see if ['pyr' string '.mat']
%            exists in the cache directory and reads that instead. 
%	
% pyr (im after prep):
%     cell array of pyr levels. each elem L:
%         L.im = (rgb or gray) uint8 image
%         L.fim =  4 int8 signed channels
%         L.scale = in octaves: new size = original size * 2^(L.lev)
%
%      examples of optional fields (anything that is 
%                  passed in in varargin...)
%         L.cent = center position [0 0], 
%         L.id = im id number 
%         L.name = string label

  ;% check to see if we were given a set of images to take the median of to
  %  create the pyramid
  if(iscell(im) & ~isstruct(im{1}))
    pyr = makePyrFromSeq(im, para, varargin{:});
    return;
  end
  
  % check if we were given a single string as a filename
  imname = 'noname';
  if(isstr(im))
    % if the string contains a wildcard, we grab the middle matching file
    if(any(im=='*'))
      dl = dir(im);
      if(length(dl)>0)
	[dn, junk, junk] = fileparts(im);
	imname = [dn '/' dl(ceil(end/2)).name];
      end
    else
      imname = im;
    end
    im = fileRead(imname);
  end
  
  % check if im is already a pyramid
  if(iscell(im))
    pyr = im;
    return;
  end

  assert(prod(size(im))>1, ['id_prepIm: Unable to Load Image ' imname]);
  
  scales = [0:para.pyr_scales(2):para.pyr_scales(1)];
  
  % set up base level
  pyr{1}.scale = 0;
  pyr{1}.im = im2uint8(im);
  
  % downsample higher levels of the pyramid... 
  %  (imresize does Gaussian smoothing)
  for i=2:length(scales)
    pyr{i}.scale = scales(i);
    pyr{i}.im = imresize(pyr{i-1}.im, 2^(-para.pyr_scales(2)), 'bilinear');
  end

  %finish each level by computing filtered image from im...
  [fb, th] = makeFilterBank(4, [5 1], [0 1], [2 4], [11 11]);
  for i=1:length(pyr)
    if(size(pyr{i}.im,3)==3)
      gim = im2double(rgb2gray(pyr{i}.im));
    else
      gim = im2double(pyr{i}.im);
    end
    
    % we store filtered images as signed 8-bit ints (range [-128 127])
    % which translates to a double range of [-0.25 0.25] 
    pyr{i}.fim = int8(filterImage(gim, fb)*128*4);
    %pyr{i}.para = para;
    
    % add in arguments such as label, im id
    pyr{i} = makeParams(pyr{i}, varargin{:});
    
  end

  
function pyr = makePyrFromSeq(im, para, varargin)
% makePyrFromSeq   make a pyramid from a sequence of aligned images of an obj
%
% pyr = makePyrFromSeq(im, para, varargin)
%
% im - cell array of images or image filenames
  
  for i=1:length(im)
    pyrset{i} = id_prepIm(im{i}, para, varargin{:});
  end
  
  error('TODO: Not Implemented. sorry...');
  
  